from hrflow import Hrflow
import requests


def workflow(settings: dict) -> None:
    """
    PULL WORKFLOW allows you to run the following code instructions on a regular basis

    @rtype: null
    @param settings: dictionary of settings params of the workflow
    """
